# Generated by Django 4.2.23 on 2025-08-08 08:42

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('pretix_wallet', '0002_alter_wallet_pan_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='Wallet_SettingsStore',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('key', models.CharField(max_length=255)),
                ('value', models.TextField()),
                ('object', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='_settings_objects', to='pretix_wallet.wallet')),
            ],
        ),
    ]
