{% extends "pretixcontrol/base.html" %}
{% load i18n %}
{% load bootstrap3 %}
{% load formset_tags %}
{% block title %}Wallet settings{% endblock %}
{% block content %}
<h1>
    Organizer wallet settings
</h1>

<form action="" method="post" class="form-horizontal" enctype="multipart/form-data">
    {% csrf_token %}
    {% bootstrap_form_errors form %}
    <fieldset>
        <legend>{% trans "General" %}</legend>
        {% bootstrap_field form.wallet_create_for_customers layout="control" %}
        {% bootstrap_field form.wallet_default_currency layout="control" %}
        {% bootstrap_field form.wallet_minimum_balance layout="control" %}
    </fieldset>
    <fieldset>
        <legend>{% trans "PANs" %}</legend>
        {% bootstrap_field form.wallet_iin layout="control" %}
        {% bootstrap_field form.wallet_pan_length layout="control" %}
    </fieldset>
    <div class="form-group submit-group">
        <button type="submit" class="btn btn-primary btn-save">
            {% trans "Save" %}
        </button>
    </div>
</form>
{% endblock %}
