# Generated by Django 4.2.23 on 2025-08-07 14:45

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('pretix_wallet', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='wallet',
            name='pan',
            field=models.CharField(db_index=True, max_length=19),
        ),
        migrations.AlterField(
            model_name='wallettransaction',
            name='order_payment',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='wallet_transactions', to='pretixbase.orderpayment'),
        ),
        migrations.AlterField(
            model_name='wallettransaction',
            name='order_position',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='wallet_transactions', to='pretixbase.orderposition'),
        ),
        migrations.AlterField(
            model_name='wallettransaction',
            name='order_refund',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='wallet_transactions', to='pretixbase.orderrefund'),
        ),
    ]
